@@ -43,8 +43,7 @@ from cassandra.protocol import (ReadyMessage, AuthenticateMessage, OptionsMessag
                                 AuthSuccessMessage, ProtocolException,
                                 RegisterMessage, ReviseRequestMessage)
 from cassandra.util import OrderedDict
-from cassandra.murmur3 import INT64_MIN
-
+from cassandra.shard_info import ShardingInfo
 
 log = logging.getLogger(__name__)
 
@@ -102,7 +101,7 @@ else:
         return snappy.decompress(byts)
     locally_supported_compressions['snappy'] = (snappy.compress, decompress)
 
-DRIVER_NAME, DRIVER_VERSION = 'DataStax Python Driver', sys.modules['cassandra'].__version__
+DRIVER_NAME, DRIVER_VERSION = 'Scylla Python Driver', sys.modules['cassandra'].__version__
 
 PROTOCOL_VERSION_MASK = 0x7f
 
@@ -215,25 +214,26 @@ class DefaultEndPointFactory(EndPointFactory):
 
     port = None
     """
-    If set, force all endpoints to use this port.
+    If no port is discovered in the row, this is the default port
+    used for endpoint creation. 
     """
 
     def __init__(self, port=None):
         self.port = port
 
     def create(self, row):
-        addr = None
-        if "rpc_address" in row:
-            addr = row.get("rpc_address")
-        if "native_transport_address" in row:
-            addr = row.get("native_transport_address")
-        if not addr or addr in ["0.0.0.0", "::"]:
-            addr = row.get("peer")
+        # TODO next major... move this class so we don't need this kind of hack
+        from cassandra.metadata import _NodeInfo
+        addr = _NodeInfo.get_broadcast_rpc_address(row)
+        port = _NodeInfo.get_broadcast_rpc_port(row)
+        if port is None:
+            port = self.port if self.port else 9042
 
         # create the endpoint with the translated address
+        # TODO next major, create a TranslatedEndPoint type
         return DefaultEndPoint(
             self.cluster.address_translator.translate(addr),
-            self.port if self.port is not None else 9042)
+            port)
 
 
 @total_ordering
@@ -600,44 +600,6 @@ if six.PY3:
 else:
     int_from_buf_item = ord
 
-class ShardingInfo(object):
-
-    def __init__(self, shard_id, shards_count, partitioner, sharding_algorithm, sharding_ignore_msb):
-        self.shards_count = int(shards_count)
-        self.partitioner = partitioner
-        self.sharding_algorithm = sharding_algorithm
-        self.sharding_ignore_msb = int(sharding_ignore_msb)
-
-    @staticmethod
-    def parse_sharding_info(message):
-        shard_id = message.options.get('SCYLLA_SHARD', [''])[0] or None
-        shards_count = message.options.get('SCYLLA_NR_SHARDS', [''])[0] or None
-        partitioner = message.options.get('SCYLLA_PARTITIONER', [''])[0] or None
-        sharding_algorithm = message.options.get('SCYLLA_SHARDING_ALGORITHM', [''])[0] or None
-        sharding_ignore_msb = message.options.get('SCYLLA_SHARDING_IGNORE_MSB', [''])[0] or None
-        log.debug("Parsing sharding info from message options %s", message.options)
-
-        if not (shard_id or shards_count or partitioner == "org.apache.cassandra.dht.Murmur3Partitioner" or
-            sharding_algorithm ==  "biased-token-round-robin" or sharding_ignore_msb):
-            return 0, None
-
-        return int(shard_id), ShardingInfo(shard_id, shards_count, partitioner, sharding_algorithm, sharding_ignore_msb)
-
-    def shard_id_from_token(self, t):
-        """
-        Convert a Murmur3 token to shard_id based on the number of shards on the host
-        """
-        token = t.value
-        token += INT64_MIN
-        token <<= self.sharding_ignore_msb
-        tokLo = token & 0xffffffff
-        tokHi = (token >> 32) & 0xffffffff
-        mul1 = tokLo * self.shards_count
-        mul2 = tokHi * self.shards_count
-        _sum = (mul1 >> 32) + mul2
-        output = _sum >> 32
-        return output
-
 
 class Connection(object):
 
@@ -733,6 +695,7 @@ class Connection(object):
         self._requests = {}
         self._iobuf = io.BytesIO()
         self._continuous_paging_sessions = {}
+        self._socket_writable = True
 
         if ssl_options:
             self._check_hostname = bool(self.ssl_options.pop('check_hostname', False))
@@ -967,6 +930,8 @@ class Connection(object):
             raise ConnectionShutdown("Connection to %s is defunct" % self.endpoint)
         elif self.is_closed:
             raise ConnectionShutdown("Connection to %s is closed" % self.endpoint)
+        elif not self._socket_writable:
+            raise ConnectionBusy("Connection %s is overloaded" % self.endpoint)
 
         # queue the decoder function with the request
         # this allows us to inject custom functions per request to encode, decode messages
@@ -1485,7 +1450,7 @@ class HeartbeatFuture(object):
         log.debug("Sending options message heartbeat on idle connection (%s) %s",
                   id(connection), connection.endpoint)
         with connection.lock:
-            if connection.in_flight <= connection.max_request_id:
+            if connection.in_flight < connection.max_request_id:
                 connection.in_flight += 1
                 connection.send_msg(OptionsMessage(), connection.get_request_id(), self._options_callback)
             else:
